﻿unit fOpenAIAssistant;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics,
  Controls, Forms, Dialogs, StdCtrls, ExtCtrls, ComCtrls,
  ImgList, Menus, ImageList, UITypes,
  // sgc
  sgcAI, sgcAI_OpenAI, sgcAI_OpenAI_Audio, sgcHTTP_API_OpenAI,
  sgcAI_OpenAI_Audio_Assistant, sgcBase_Classes;

type
  TsgcAssistantIcons = (iconPlay, iconStop);

  TFRMOpenAIAssistant = class(TForm)
    pnlClient: TPanel;
    memoLog: TMemo;
    pageMain: TPageControl;
    tabLog: TTabSheet;
    tabConfiguration: TTabSheet;
    pageConfiguration: TPageControl;
    pnlConfigurationTop: TPanel;
    Label1: TLabel;
    txtAPIKey: TEdit;
    pnlConfigurationClient: TPanel;
    Assistant: TsgcAIOpenAIAssistant;
    tabAssistant: TTabSheet;
    txtAssistantName: TEdit;
    Label2: TLabel;
    memoAssistantInstructions: TMemo;
    Label3: TLabel;
    Label4: TLabel;
    txtAssistantModel: TEdit;
    chkAssistantFileSearch: TCheckBox;
    chkAssistantCodeInterpreter: TCheckBox;
    trackAssistantTemperature: TTrackBar;
    Label5: TLabel;
    Label6: TLabel;
    trackAssistantTopP: TTrackBar;
    pnlTop: TPanel;
    btnSendMessage: TButton;
    memoMessage: TMemo;
    Label7: TLabel;
    btnCreateAssistant: TButton;
    cboAssistant: TComboBox;
    Label8: TLabel;
    txtVectorStore: TEdit;
    Label9: TLabel;
    cboVectorStoreFiles: TComboBox;
    Label10: TLabel;
    btnVectorStoreUploadFile: TButton;
    procedure btnCreateAssistantClick(Sender: TObject);
    procedure btnSendMessageClick(Sender: TObject);
    procedure btnVectorStoreUploadFileClick(Sender: TObject);
    procedure cboAssistantChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FAssistant: TsgcOpenAIClass_Assistant;
    FThread: TsgcOpenAIClass_Thread;
    FAssistantId: string;
    procedure DoLog(const aText: string);
  private
    procedure DoLoadSettings;
    procedure DoSaveSettings;
  private
    function DoFormatResponse(const aText: string): string;
    function DoCreateAssistant: Boolean;
    procedure DoUpdateVectorStoreFiles;
  end;

var
  FRMOpenAIAssistant: TFRMOpenAIAssistant;

implementation

{$R *.dfm}

uses
  INIFiles, StrUtils, sgcBase_Helpers;

procedure TFRMOpenAIAssistant.btnCreateAssistantClick(Sender: TObject);
begin
  FAssistantId := '';
  if DoCreateAssistant then
    pageMain.ActivePage := tabLog;
end;

procedure TFRMOpenAIAssistant.btnSendMessageClick(Sender: TObject);
var
  i: Integer;
  oMessage: TsgcOpenAIClass_Message;
  oMessages: TsgcOpenAIClass_Response_List_Messages;
  oRun: TsgcOpenAIClass_Run;
begin
  if not Assigned(FAssistant) then
    raise Exception.Create('Before send a message create an Assistant!!!');
  if not Assigned(FThread) then
    raise Exception.Create('Before send a message create a Thread!!!');

  DoLog('[user]: ' + memoMessage.Lines.Text);
  Screen.Cursor := crHourGlass;
  Try
    oMessage := Assistant.CreateMessageText(FThread.Id, memoMessage.Lines.Text);
    if Assigned(oMessage) then
    begin
      oRun := Assistant.CreateRunAndWait(FThread.Id);
      if Assigned(oRun) then
      begin
        oMessages := Assistant.GetMessages(FThread.Id, oRun.Id);
        if Assigned(oMessages) and (Length(oMessages.Messages) > 0) then
        begin
          memoMessage.Lines.Text := '';
          for i := 0 to Length(oMessages.Messages) - 1 do
            DoLog('[assistant]: ' + DoFormatResponse(oMessages.Messages[i]
              .ContentText + #13#10));
        end;
      end;
    end;
  Finally
    Screen.Cursor := crDefault;
  End;
end;

procedure TFRMOpenAIAssistant.btnVectorStoreUploadFileClick(Sender: TObject);
var
  oDialog: TOpenDialog;
begin
  if FAssistantId = '' then
    raise Exception.Create('Create First an Assistant!!!');

  oDialog := TOpenDialog.Create(nil);
  Try
    if oDialog.Execute then
    begin
      Screen.Cursor := crHourGlass;
      Try
        Assistant.UploadVectorStoreFile(txtVectorStore.Text, oDialog.FileName);
        DoUpdateVectorStoreFiles();
        FThread := Assistant.CreateThread;
      Finally
        Screen.Cursor := crDefault;
      End;
    end;
  Finally
    oDialog.Free;
  End;
end;

procedure TFRMOpenAIAssistant.cboAssistantChange(Sender: TObject);
begin
  case cboAssistant.ItemIndex of
    0:
      begin
        txtAssistantName.Text := 'Delphi Math Tutor';
        memoAssistantInstructions.Lines.Text :=
          'You are a personal math tutor. ' +
          'Answer questions briefly, in a sentence or less. When asked a question, use Delphi'
          + 'code to answer the question.';
        chkAssistantFileSearch.Checked := False;
        chkAssistantCodeInterpreter.Checked := True;
        memoMessage.Lines.Text :=
          'I need to solve the equation `3x + 11 = 14`. Can you help me?';
      end;
    1:
      begin
        txtAssistantName.Text := 'sgcWebSockets HelpDesk';
        memoAssistantInstructions.Lines.Text :=
          'You are a sgcWebSockets HelpDesk Agent. ' +
          'Answer questions briefly, in a sentence or less. When asked a question, '
          + 'use the manual to answer the question.';
        chkAssistantFileSearch.Checked := True;
        chkAssistantCodeInterpreter.Checked := False;
        memoMessage.Lines.Text :=
          'Create a Delphi WebSocketClient that connects ' +
          'to eSeGeCe WebSocket server and when the message is received it echoes to the server.';
      end;
  end;
end;

function TFRMOpenAIAssistant.DoFormatResponse(const aText: string): string;
begin
  result := aText;
  result := sgcStringReplace(result, '\n', #13#10);
  result := sgcStringReplace(result, '\\', '');
end;

procedure TFRMOpenAIAssistant.FormDestroy(Sender: TObject);
begin
  DoSaveSettings;
end;

procedure TFRMOpenAIAssistant.FormCreate(Sender: TObject);
begin
  Try
    // settings
    DoLoadSettings;
    // visible
    if txtAPIKey.Text = '' then
    begin
      pageMain.ActivePage := tabConfiguration;
    end
    else
    begin
      pageMain.ActivePage := tabLog;
      DoCreateAssistant;
    end;
  Except
    On E: Exception do
    begin
      MessageDlg('Initialization error: ' + E.Message, mtError, [mbOk],
        0, mbOk);
      Application.Terminate;
    end;
  End;
end;

procedure TFRMOpenAIAssistant.DoLoadSettings;
var
  oINI: TINIFile;
begin
  oINI := TINIFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  Try
    txtAPIKey.Text := oINI.ReadString('OPENAI', 'API_KEY', '');

    // assistant
    FAssistantId := oINI.ReadString('ASSISTANT', 'ID', '');
    cboAssistant.ItemIndex := oINI.ReadInteger('ASSISTANT', 'TYPE', 0);
    cboAssistantChange(cboAssistant);
    txtAssistantName.Text := oINI.ReadString('ASSISTANT', 'NAME',
      txtAssistantName.Text);
    memoAssistantInstructions.Lines.Text := oINI.ReadString('ASSISTANT',
      'INSTRUCTIONS', memoAssistantInstructions.Lines.Text);
    txtAssistantModel.Text := oINI.ReadString('ASSISTANT', 'MODEL',
      txtAssistantModel.Text);
    chkAssistantFileSearch.Checked := oINI.ReadBool('ASSISTANT', 'FILE_SEARCH',
      chkAssistantFileSearch.Checked);
    chkAssistantCodeInterpreter.Checked := oINI.ReadBool('ASSISTANT',
      'CODE_INTERPRETER', chkAssistantCodeInterpreter.Checked);
    trackAssistantTemperature.Position := oINI.ReadInteger('ASSISTANT',
      'TEMPERATURE', trackAssistantTemperature.Position);
    trackAssistantTopP.Position := oINI.ReadInteger('ASSISTANT', 'TOPP',
      trackAssistantTopP.Position);
  Finally
    oINI.Free;
  End;
end;

procedure TFRMOpenAIAssistant.DoLog(const aText: string);
begin
  memoLog.Lines.Add(aText);
end;

procedure TFRMOpenAIAssistant.DoSaveSettings;
var
  oINI: TINIFile;
begin
  oINI := TINIFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  Try
    oINI.Writestring('OPENAI', 'API_KEY', txtAPIKey.Text);

    // assistant
    if Assigned(FAssistant) then
      oINI.Writestring('ASSISTANT', 'ID', FAssistant.Id)
    else
      oINI.Writestring('ASSISTANT', 'ID', '');
    oINI.WriteInteger('ASSISTANT', 'TYPE', cboAssistant.ItemIndex);
    oINI.Writestring('ASSISTANT', 'NAME', txtAssistantName.Text);
    oINI.Writestring('ASSISTANT', 'INSTRUCTIONS',
      memoAssistantInstructions.Lines.Text);
    oINI.Writestring('ASSISTANT', 'MODEL', txtAssistantModel.Text);
    oINI.WriteBool('ASSISTANT', 'FILE_SEARCH', chkAssistantFileSearch.Checked);
    oINI.WriteBool('ASSISTANT', 'CODE_INTERPRETER',
      chkAssistantCodeInterpreter.Checked);
    oINI.WriteInteger('ASSISTANT', 'TEMPERATURE',
      trackAssistantTemperature.Position);
    oINI.WriteInteger('ASSISTANT', 'TOPP', trackAssistantTopP.Position);
  Finally
    oINI.Free;
  End;
end;

function TFRMOpenAIAssistant.DoCreateAssistant: Boolean;
begin
  if txtAPIKey.Text = '' then
    raise Exception.Create('Set First the OpenAI API Key!!!');

  Assistant.OpenAIOptions.ApiKey := txtAPIKey.Text;

  Assistant.AssistantOptions.Name := txtAssistantName.Text;
  Assistant.AssistantOptions.Instructions.Text :=
    memoAssistantInstructions.Lines.Text;
  Assistant.AssistantOptions.Model := txtAssistantModel.Text;
  Assistant.AssistantOptions.Tools.FileSearch.Enabled :=
    chkAssistantFileSearch.Checked;
  Assistant.AssistantOptions.Tools.CodeInterpreter.Enabled :=
    chkAssistantCodeInterpreter.Checked;
  Assistant.AssistantOptions.Temperature := trackAssistantTemperature.Position;
  Assistant.AssistantOptions.TopP := trackAssistantTopP.Position;

  if FAssistantId = '' then
  begin
    FAssistant := Assistant.CreateAssistant;
    if Assigned(FAssistant) then
    begin
      FThread := Assistant.CreateThread;
      DoLog('#create assistant: ' + FAssistant.Id);
    end;
  end
  else
  begin
    FAssistant := Assistant.GetAssistant(FAssistantId);
    if Assigned(FAssistant) then
      FThread := Assistant.CreateThread;
  end;
  DoUpdateVectorStoreFiles;

  if Assigned(FAssistant) then
    FAssistantId := FAssistant.Id;

  result := (FAssistantId <> '') and Assigned(FThread);
  if result and (cboAssistant.ItemIndex = 1)  then
    result := cboVectorStoreFiles.Items.Count > 0;
end;

procedure TFRMOpenAIAssistant.DoUpdateVectorStoreFiles;
var
  i: Integer;
  oVectorStoreFile: TsgcOpenAIClass_VectorStoreFile;
  oVectorStoreFiles: TsgcOpenAIClass_Response_List_VectorStoreFiles;
begin
  cboVectorStoreFiles.Items.Clear;
  oVectorStoreFiles := Assistant.GetVectorStoreFiles(txtVectorStore.Text);
  if Assigned(oVectorStoreFiles) then
  begin
    for i:=Low(oVectorStoreFiles.VectorStoreFiles) to High(oVectorStoreFiles.VectorStoreFiles) do
    begin
      oVectorStoreFile := oVectorStoreFiles.VectorStoreFiles[i];
      cboVectorStoreFiles.Items.Add(oVectorStoreFile.Id);
    end;
  end;
end;

end.
